@@ -31,7 +31,7 @@
  * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  * SOFTWARE.
  *
- * $Id: cmd.c 5882 2006-03-17 23:55:09Z roland $
+ * $Id: cmd.c 10015 2006-10-31 15:52:42Z roland $
  */
 
 #if HAVE_CONFIG_H
@@ -72,6 +72,8 @@ static int ibv_cmd_get_context_v2(struct ibv_context *context,
 	if (write(context->cmd_fd, cmd, cmd_size) != cmd_size)
 		return errno;
 
+	VALGRIND_MAKE_MEM_DEFINED(resp, resp_size);
+
 	context->async_fd         = resp->async_fd;
 	context->num_comp_vectors = 1;
 	t->channel.fd		  = cq_fd;
@@ -92,6 +94,8 @@ int ibv_cmd_get_context(struct ibv_context *context, struct ibv_get_context *cmd
 	if (write(context->cmd_fd, cmd, cmd_size) != cmd_size)
 		return errno;
 
+	VALGRIND_MAKE_MEM_DEFINED(resp, resp_size);
+
 	context->async_fd         = resp->async_fd;
 	context->num_comp_vectors = resp->num_comp_vectors;
 
@@ -110,6 +114,8 @@ int ibv_cmd_query_device(struct ibv_context *context,
 	if (write(context->cmd_fd, cmd, cmd_size) != cmd_size)
 		return errno;
 
+	VALGRIND_MAKE_MEM_DEFINED(&resp, sizeof resp);
+
 	memset(device_attr->fw_ver, 0, sizeof device_attr->fw_ver);
 	*raw_fw_ver			       = resp.fw_ver;
 	device_attr->node_guid 		       = resp.node_guid;
@@ -163,10 +169,13 @@ int ibv_cmd_query_port(struct ibv_context *context, uint8_t port_num,
 
 	IBV_INIT_CMD_RESP(cmd, cmd_size, QUERY_PORT, &resp, sizeof resp);
 	cmd->port_num = port_num;
+	memset(cmd->reserved, 0, sizeof cmd->reserved);
 
 	if (write(context->cmd_fd, cmd, cmd_size) != cmd_size)
 		return errno;
 
+	VALGRIND_MAKE_MEM_DEFINED(&resp, sizeof resp);
+
 	port_attr->state      	   = resp.state;
 	port_attr->max_mtu         = resp.max_mtu;
 	port_attr->active_mtu      = resp.active_mtu;
@@ -199,6 +208,8 @@ int ibv_cmd_alloc_pd(struct ibv_context *context, struct ibv_pd *pd,
 	if (write(context->cmd_fd, cmd, cmd_size) != cmd_size)
 		return errno;
 
+	VALGRIND_MAKE_MEM_DEFINED(resp, resp_size);
+
 	pd->handle = resp->pd_handle;
 
 	return 0;
@@ -235,6 +246,8 @@ int ibv_cmd_reg_mr(struct ibv_pd *pd, void *addr, size_t length,
 	if (write(pd->context->cmd_fd, cmd, cmd_size) != cmd_size)
 		return errno;
 
+	VALGRIND_MAKE_MEM_DEFINED(&resp, sizeof resp);
+
 	mr->handle  = resp.mr_handle;
 	mr->lkey    = resp.lkey;
 	mr->rkey    = resp.rkey;
@@ -275,6 +288,8 @@ static int ibv_cmd_create_cq_v2(struct ibv_context *context, int cqe,
 	if (write(context->cmd_fd, cmd, cmd_size) != cmd_size)
 		return errno;
 
+	VALGRIND_MAKE_MEM_DEFINED(resp, sizeof resp_size);
+
 	cq->handle = resp->cq_handle;
 	cq->cqe    = resp->cqe;
 
@@ -301,6 +316,8 @@ int ibv_cmd_create_cq(struct ibv_context *context, int cqe,
 	if (write(context->cmd_fd, cmd, cmd_size) != cmd_size)
 		return errno;
 
+	VALGRIND_MAKE_MEM_DEFINED(resp, resp_size);
+
 	cq->handle = resp->cq_handle;
 	cq->cqe    = resp->cqe;
 
@@ -329,6 +346,8 @@ int ibv_cmd_poll_cq(struct ibv_cq *ibcq, int ne, struct ibv_wc *wc)
 		goto out;
 	}
 
+	VALGRIND_MAKE_MEM_DEFINED(resp, rsize);
+
 	for (i = 0; i < resp->count; i++) {
 		wc[i].wr_id 	     = resp->wc[i].wr_id;
 		wc[i].status 	     = resp->wc[i].status;
@@ -378,6 +397,8 @@ int ibv_cmd_resize_cq(struct ibv_cq *cq, int cqe,
 	if (write(cq->context->cmd_fd, cmd, cmd_size) != cmd_size)
 		return errno;
 
+	VALGRIND_MAKE_MEM_DEFINED(&resp, sizeof resp);
+
 	cq->cqe = resp.cqe;
 
 	return 0;
@@ -410,6 +431,8 @@ int ibv_cmd_destroy_cq(struct ibv_cq *cq)
 	if (write(cq->context->cmd_fd, &cmd, sizeof cmd) != sizeof cmd)
 		return errno;
 
+	VALGRIND_MAKE_MEM_DEFINED(&resp, sizeof resp);
+
 	pthread_mutex_lock(&cq->mutex);
 	while (cq->comp_events_completed  != resp.comp_events_reported ||
 	       cq->async_events_completed != resp.async_events_reported)
@@ -434,6 +457,8 @@ int ibv_cmd_create_srq(struct ibv_pd *pd,
 	if (write(pd->context->cmd_fd, cmd, cmd_size) != cmd_size)
 		return errno;
 
+	VALGRIND_MAKE_MEM_DEFINED(resp, resp_size);
+
 	srq->handle = resp->srq_handle;
 
 	if (abi_ver > 5) {
@@ -574,10 +599,13 @@ int ibv_cmd_create_qp(struct ibv_pd *pd,
 	cmd->sq_sig_all	     = attr->sq_sig_all;
 	cmd->qp_type 	     = attr->qp_type;
 	cmd->is_srq 	     = !!attr->srq;
+	cmd->reserved	     = 0;
 
 	if (write(pd->context->cmd_fd, cmd, cmd_size) != cmd_size)
 		return errno;
 
+	VALGRIND_MAKE_MEM_DEFINED(resp, resp_size);
+
 	qp->handle 		  = resp->qp_handle;
 	qp->qp_num 		  = resp->qpn;
 
@@ -633,7 +661,7 @@ int ibv_cmd_query_qp(struct ibv_qp *qp, struct ibv_qp_attr *attr,
 	attr->cur_qp_state                  = resp.cur_qp_state;
 	attr->path_mtu                      = resp.path_mtu;
 	attr->path_mig_state                = resp.path_mig_state;
-	attr->en_sqd_async_notify           = resp.en_sqd_async_notify;
+	attr->sq_draining                   = resp.sq_draining;
 	attr->max_rd_atomic                 = resp.max_rd_atomic;
 	attr->max_dest_rd_atomic            = resp.max_dest_rd_atomic;
 	attr->min_rnr_timer                 = resp.min_rnr_timer;
@@ -742,6 +770,8 @@ int ibv_cmd_modify_qp(struct ibv_qp *qp, struct ibv_qp_attr *attr,
 	cmd->alt_dest.is_global     = attr->alt_ah_attr.is_global;
 	cmd->alt_dest.port_num 	    = attr->alt_ah_attr.port_num;
 
+	cmd->reserved[0] = cmd->reserved[1] = 0;
+
 	if (write(qp->context->cmd_fd, cmd, cmd_size) != cmd_size)
 		return errno;
 
@@ -837,6 +867,8 @@ int ibv_cmd_post_send(struct ibv_qp *ibqp, struct ibv_send_wr *wr,
 	if (write(ibqp->context->cmd_fd, cmd, cmd_size) != cmd_size)
 		ret = errno;
 
+	VALGRIND_MAKE_MEM_DEFINED(&resp, sizeof resp);
+
 	wr_count = resp.bad_wr;
 	if (wr_count) {
 		i = wr;
@@ -895,6 +927,8 @@ int ibv_cmd_post_recv(struct ibv_qp *ibqp, struct ibv_recv_wr *wr,
 	if (write(ibqp->context->cmd_fd, cmd, cmd_size) != cmd_size)
 		ret = errno;
 
+	VALGRIND_MAKE_MEM_DEFINED(&resp, sizeof resp);
+
 	wr_count = resp.bad_wr;
 	if (wr_count) {
 		i = wr;
@@ -953,6 +987,8 @@ int ibv_cmd_post_srq_recv(struct ibv_srq *srq, struct ibv_recv_wr *wr,
 	if (write(srq->context->cmd_fd, cmd, cmd_size) != cmd_size)
 		ret = errno;
 
+	VALGRIND_MAKE_MEM_DEFINED(&resp, sizeof resp);
+
 	wr_count = resp.bad_wr;
 	if (wr_count) {
 		i = wr;
@@ -988,6 +1024,8 @@ int ibv_cmd_create_ah(struct ibv_pd *pd, struct ibv_ah *ah,
 	if (write(pd->context->cmd_fd, &cmd, sizeof cmd) != sizeof cmd)
 		return errno;
 
+	VALGRIND_MAKE_MEM_DEFINED(&resp, sizeof resp);
+
 	ah->handle = resp.handle;
 
 	return 0;
@@ -1020,6 +1058,8 @@ int ibv_cmd_destroy_qp(struct ibv_qp *qp)
 	if (write(qp->context->cmd_fd, &cmd, sizeof cmd) != sizeof cmd)
 		return errno;
 
+	VALGRIND_MAKE_MEM_DEFINED(&resp, sizeof resp);
+
 	pthread_mutex_lock(&qp->mutex);
 	while (qp->events_completed != resp.events_reported)
 		pthread_cond_wait(&qp->cond, &qp->mutex);
