@@ -29,7 +29,7 @@
  * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  * SOFTWARE.
  *
- * $Id: device.c 6432 2006-04-11 23:27:38Z roland $
+ * $Id: device.c 7621 2006-06-01 23:16:18Z roland $
  */
 
 #if HAVE_CONFIG_H
@@ -63,7 +63,7 @@ struct ibv_device **ibv_get_device_list(int *num)
 	if (!num_devices)
 		num_devices = ibverbs_init(&device_list);
 
-	l = calloc(num_devices, sizeof (struct ibv_device *));
+	l = calloc(num_devices + 1, sizeof (struct ibv_device *));
 	for (i = 0; i < num_devices; ++i)
 		l[i] = device_list[i];
 
@@ -119,6 +119,8 @@ struct ibv_context *ibv_open_device(struct ibv_device *device)
 	 * provider needs to mmap() the file.
 	 */
 	cmd_fd = open(devpath, O_RDWR);
+	free(devpath);
+
 	if (cmd_fd < 0)
 		return NULL;
 
